package com.hb.util;

import java.io.InputStream;
import java.util.*;
import java.util.concurrent.TimeUnit;

import io.minio.*;
import io.minio.http.Method;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Component;

import com.hb.domain.Fileinfo;
import io.minio.messages.Bucket;
import io.minio.messages.Item;
import org.springframework.web.multipart.MultipartFile;

@Component
public class MinioUtil {
	@Autowired
    private MinioClient minioClient;

	@Autowired
	private RedisTemplate redisTemplate;

	/**
	 * 创建一个桶
	 */
	public void createBucket(String bucket) throws Exception {
		boolean found = minioClient.bucketExists(BucketExistsArgs.builder().bucket(bucket).build());
		if (!found) {
			minioClient.makeBucket(MakeBucketArgs.builder().bucket(bucket).build());
		}
	}

	/**
	 * 上传一个文件
	 */
	public void uploadFile(InputStream stream, String bucket, String objectName) throws Exception {
		minioClient.putObject(PutObjectArgs.builder().bucket(bucket).object(objectName)
				.stream(stream, -1, 10485760).build());
	}



	/**
	 * 列出所有的桶
	 */
	public List<String> listBuckets() throws Exception {
		List<Bucket> list = minioClient.listBuckets();
		List<String> names = new ArrayList<>();
		list.forEach(b -> {
			names.add(b.name());
		});
		return names;
	}

	/**
	 * 列出一个桶中的所有文件和目录
	 */
	public List<Fileinfo> listFiles(String bucket) throws Exception {
		Iterable<Result<Item>> results = minioClient.listObjects(
			    ListObjectsArgs.builder().bucket(bucket).recursive(true).build());

			List<Fileinfo> infos = new ArrayList<>();
				results.forEach(r->{
					Fileinfo info = new Fileinfo();
					try {
						Item item = r.get();
						info.setFilename(item.objectName());
						info.setDirectory(item.isDir());
						infos.add(info);
					} catch (Exception e) {
						e.printStackTrace();
					}
				});
		return infos;
	}

	/**
	 * 下载一个文件
	 */
	public InputStream download(String bucket, String objectName) throws Exception {
		InputStream stream = minioClient.getObject(
		              GetObjectArgs.builder().bucket(bucket).object(objectName).build());
		return stream;
	}

	/**
	 * 删除一个桶
	 */
	public void deleteBucket(String bucket) throws Exception {
		minioClient.removeBucket(RemoveBucketArgs.builder().bucket(bucket).build());
	}

	/**
	 * 删除一个对象
	 */
	public void deleteObject(String bucket, String objectName) throws Exception {
		minioClient.removeObject(RemoveObjectArgs.builder().bucket(bucket).object(objectName).build());
	}

	/**
	 * 生成直传链接
	 * @param fileDirName
	 * @param bucket
	 * @return
	 */
	public String getOneURL(String fileDirName, String bucket) throws Exception{
		boolean found = minioClient.bucketExists(BucketExistsArgs.builder().bucket(bucket).build());
		if (!found) {
			throw new RuntimeException("桶不存在！");
		}

		Map<String, String> reqParams = new HashMap<String, String>();
		reqParams.put("response-content-type", "application/json");
		return minioClient.getPresignedObjectUrl(
				GetPresignedObjectUrlArgs.builder()
						.method(Method.PUT)     //这里必须是PUT，如果是GET的话就是文件访问地址了。如果是POST上传会报错.
						.bucket(bucket)         // 存储桶
						.object(fileDirName)    // 文件名
						.expiry(60 * 60 * 24)
						.extraQueryParams(reqParams)
						.build());
	}

	public String getMinioURL(String bucket, String fileDirName) throws Exception{
		String key = bucket + ":" + fileDirName.replaceAll("/", ":");
		String url = (String) redisTemplate.opsForValue().get(key);
		if (StringUtils.isEmpty(url)) {
			int timeout = 60 * 60 * 24 * 7;
			GetPresignedObjectUrlArgs build = GetPresignedObjectUrlArgs.builder()
					.method(Method.GET)
					.bucket(bucket)
					.object(fileDirName)
					.expiry(timeout, TimeUnit.SECONDS)   //生成的预签名url可访问的有效时间，最大期限7天
					.build();
			url = minioClient.getPresignedObjectUrl(build);
			redisTemplate.opsForValue().set(key, url);
			redisTemplate.expire(key, timeout, TimeUnit.SECONDS);
		}
		return url;
	}

	public List<String> getMinioURLs(String bucket, String[] fileDirNames) throws Exception{
		List<String> urlList = new ArrayList<>();
		for (String fileDirName : fileDirNames) {
			GetPresignedObjectUrlArgs build = GetPresignedObjectUrlArgs.builder()
					.method(Method.GET)
					.bucket(bucket)
					.object(fileDirName)
					.expiry(60 * 60 * 24 * 7)   //生成的预签名url可访问的有效时间，最大期限7天
					.build();
			String url = minioClient.getPresignedObjectUrl(build);
			urlList.add(url);
		}
		return urlList;
	}

	public String getContenType(String bucket, String objectName) throws Exception{
		// 创建StatObjectArgs对象
		StatObjectArgs statObjectArgs = StatObjectArgs.builder()
				.bucket(bucket)
				.object(objectName)
				.build();

		// 获取对象信息
		StatObjectResponse statObjectResponse = minioClient.statObject(statObjectArgs);

		if (statObjectResponse == null) {
			throw  new RuntimeException("文件不存在！");
		}

		// 获取文件类型
		String contentType = statObjectResponse.contentType();
		System.out.println("文件类型：" + contentType);
		return contentType;
	}

	public void uploadFile2(MultipartFile file, String bucket, String fileDirName) throws Exception{

		Long fileSize = file.getSize();
		String fileType = file.getContentType();
		String fileName = file.getOriginalFilename();

		minioClient.putObject(PutObjectArgs.builder()
				.bucket(bucket)                                // 存储桶
				.object(fileDirName)                           // 文件名
				.stream(file.getInputStream(), fileSize, -1) // 文件内容
				.contentType(fileType)                         // 文件类型
				.build());
	}

}
